#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed May 26 21:15:44 2021

@author: liran
"""

# -*- coding: utf-8 -*-
import cv2
import numpy as np
import argparse
from first import Ui_image   # 导入生成first.py里生成的类
import sys
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import *
from PyQt5.QtWidgets import QFileDialog, QMessageBox, QDockWidget, QListWidget
from PyQt5.QtGui import *

def Gaussian_Blur(gray):  # 高斯去噪(去除图像中的噪点)

    blurred = cv2.GaussianBlur(gray, (9, 9), 0)
    return blurred

def Sobel_gradient(blurred):  #索比尔算子来计算x、y方向梯度

    gradX = cv2.Sobel(blurred, ddepth=cv2.CV_32F, dx=1, dy=0)
    gradY = cv2.Sobel(blurred, ddepth=cv2.CV_32F, dx=0, dy=1)
    gradient = cv2.subtract(gradX, gradY)
    gradient = cv2.convertScaleAbs(gradient)
    return gradX, gradY, gradient


def Thresh_and_blur(gradient,thd=-1):  # 设定阈值
    blurred = cv2.GaussianBlur(gradient, (9, 9), 0)
    if(thd>0):
        (threshold, thresh) = cv2.threshold(blurred, thd, 255, cv2.THRESH_BINARY)
    else:
        (threshold, thresh) = cv2.threshold(blurred, 0, 255, cv2.THRESH_OTSU)
    return thresh,threshold


def image_morphology(thresh):
    """
     用椭圆核函数进行形态学变换
    """
    kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (25, 25))
    closed = cv2.morphologyEx(thresh, cv2.MORPH_CLOSE, kernel)
    closed = cv2.erode(closed, None, iterations=4)
    closed = cv2.dilate(closed, None, iterations=4)
    return closed


def findcnts_and_box_point(closed):# 这里opencv3返回的是三个参数


    (_, cnts, _) = cv2.findContours(closed.copy(),
                                    cv2.RETR_LIST,
                                    cv2.CHAIN_APPROX_SIMPLE)
    c = sorted(cnts, key=cv2.contourArea, reverse=True)[0]
    # 计算最大轮廓的旋转包围盒
    rect = cv2.minAreaRect(c)
    box = np.int0(cv2.boxPoints(rect))
    return box


def drawcnts_and_cut(original_img, box):  # 目标图像裁剪

    draw_img = cv2.drawContours(original_img.copy(), [box], -1, (0, 0, 255), 3)

    Xs = [i[0] for i in box]
    Ys = [i[1] for i in box]
    x1 = min(Xs)
    x2 = max(Xs)
    y1 = min(Ys)
    y2 = max(Ys)
    hight = y2 - y1
    width = x2 - x1
    crop_img = original_img[y1:y1 + hight, x1:x1 + width]
    return draw_img, crop_img


def cut(image,thd=-1):#返回方框图像

    original_img = image
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    blurred = Gaussian_Blur(gray)
    gradX, gradY, gradient = Sobel_gradient(blurred)
    thresh,threshold = Thresh_and_blur(gradient,thd)
    closed = image_morphology(thresh)
    box = findcnts_and_box_point(closed)
    draw_img, crop_img = drawcnts_and_cut(original_img, box)

    return crop_img,threshold

def segmentation(image,thd=-1):#返回切割好的图像
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    #blurred = Gaussian_Blur(gray)
    #gradX, gradY, gradient = Sobel_gradient(blurred)
    thresh,threshold = Thresh_and_blur(gray,thd)
    closed = image_morphology(thresh)
    b, g, r = cv2.split(image)
    rows = b.shape[0]
    cols = b.shape[1]
    k=0
    mask=0

    for i in range(rows):
        for j in range(cols):
            if(thresh[i][j]<1):
                k=k+1
    if(k<rows*cols/2):
        for i in range(rows):
            for j in range(cols):
                if(thresh[i][j]>0):
                    r[i][j] = 0
                    g[i][j] = 0
                    b[i][j] = 0
    else:
        for i in range(rows):
            for j in range(cols):
                if(thresh[i][j]<1):
                    r[i][j] = 0
                    g[i][j] = 0
                    b[i][j] = 0
    img=cv2.merge([r, g, b])
    return img,threshold
def binaryzation(image,thd=-1):#二值化后图片
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    #blurred = Gaussian_Blur(gray)
    #gradX, gradY, gradient = Sobel_gradient(blurred)
    thresh,threshold = Thresh_and_blur(gray,thd)
    closed = image_morphology(thresh)
    return thresh,threshold

class mywindow(QtWidgets.QWidget,Ui_image):

    def __init__(self):
        super(mywindow,self).__init__()
        self.setupUi(self)
        self.spinBox.setRange(1, 254)
        self.radioButton.setCheckable(True)
        self.imgName=""
        #定义槽函数
    def openimage(self):
   # 打开文件路径
   #设置文件扩展名过滤,注意用双分号间隔
        self.imgName,imgType= QFileDialog.getOpenFileName(self,
                                    "打开图片",
                                    "",
                                    " *.jpg;;*.png;;*.jpeg;;*.bmp;;All Files (*)")

        print(self.imgName)
        #利用qlabel显示图片
        png = QtGui.QPixmap(self.imgName).scaled(self.label.width(), self.label.height())
        self.label.setPixmap(png)
    def cutimage(self):
        if (self.imgName == ""):
            button = QMessageBox.warning(self, "Warning", "请先选择图片",
                                         QMessageBox.Ok | QMessageBox.Cancel, QMessageBox.Ok)
            return
        #print(self.imgName)
        #png=cv2.imread('C:/Users/xy-chen/Pictures/chongzi.png')
        png = cv2.imread(self.imgName)
        if(self.radioButton.isChecked()):
            img,threshold=cut(png)
        else:
            k=self.spinBox.value()
            img, threshold = cut(png,k)
        img = cv2.resize(img, (self.label_2.width(), self.label_2.height()))
        img_rgb=cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        image_height, image_width, image_depth = img_rgb.shape
        #QIm = QtGui.QImage(img_rgb.data, img_rgb.shape[1], img_rgb.shape[0], QtGui.QImage.Format_RGB32)
        QIm = QtGui.QImage(img_rgb.data, image_width, image_height, image_width*3,QtGui.QImage.Format_RGB888)
        #print(self.imgName)
        #self.label.resize(QtCore.QSize(img_rgb.shape[1], img_rgb.shape[0]))
        self.label_2.setPixmap(QPixmap.fromImage(QIm))
        self.label_7.setText(str(threshold))
        self.spinBox.setValue(int(threshold))
    def binaryzationimage(self):
        if (self.imgName == ""):
            button = QMessageBox.warning(self, "Warning", "请先选择图片",
                                         QMessageBox.Ok | QMessageBox.Cancel, QMessageBox.Ok)
            return
        png = cv2.imread(self.imgName)
        if (self.radioButton.isChecked()):

            img,threshold = binaryzation(png)
        else:
            k = self.spinBox.value()
            img, threshold = binaryzation(png,k)
        img = cv2.resize(img, (self.label_2.width(), self.label_2.height()))
        img_rgb = cv2.cvtColor(img, cv2.COLOR_GRAY2RGB)
        image_height, image_width, image_depth = img_rgb.shape
        # QIm = QtGui.QImage(img_rgb.data, img_rgb.shape[1], img_rgb.shape[0], QtGui.QImage.Format_RGB32)
        QIm = QtGui.QImage(img_rgb.data, image_width, image_height, image_width * 3, QtGui.QImage.Format_RGB888)
        #QIm = QtGui.QImage(img.data, image_width, image_height, image_width * 3, QtGui.QImage.Format_RGB888)
        # print(self.imgName)
        # self.label.resize(QtCore.QSize(img_rgb.shape[1], img_rgb.shape[0]))
        self.label_2.setPixmap(QPixmap.fromImage(QIm))
        self.label_7.setText(str(threshold))
        self.spinBox.setValue(int(threshold))
    def segmentationimage(self):
        if(self.imgName == ""):
            button = QMessageBox.warning(self, "Warning", "请先选择图片",
                                         QMessageBox.Ok | QMessageBox.Cancel, QMessageBox.Ok)
            return
        png = cv2.imread(self.imgName)
        if (self.radioButton.isChecked()):

            img,threshold = segmentation(png)
        else:
            k = self.spinBox.value()
            img, threshold = segmentation(png,k)
        img = cv2.resize(img, (self.label_2.width(), self.label_2.height()))
        img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        image_height, image_width, image_depth = img_rgb.shape
        # QIm = QtGui.QImage(img_rgb.data, img_rgb.shape[1], img_rgb.shape[0], QtGui.QImage.Format_RGB32)
        QIm = QtGui.QImage(img_rgb.data, image_width, image_height, image_width * 3, QtGui.QImage.Format_RGB888)
        # print(self.imgName)
        # self.label.resize(QtCore.QSize(img_rgb.shape[1], img_rgb.shape[0]))
        self.label_2.setPixmap(QPixmap.fromImage(QIm))
        self.label_7.setText(str(threshold))
        self.spinBox.setValue(int(threshold))


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    window = mywindow()
    window.show()
    sys.exit(app.exec_())
